/*
 * config.c
 * 
 * $Revision: 15 $
 * $Date: 2006-07-22 16:46:05 +0200 (So, 22 čec 2006) $
 * 
 * Copyright (C) 2006  Josef Kufner <jk@myserver.cz>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */
#include <stdlib.h>
#include <libgen.h>
#include <string.h>
#include <unistd.h>
#include <assert.h>

#include "config.h"
#include "parser.h"

#define DEBUG 0
#include "debug.h"

struct s_cfg cfg = {
	/* win_title */		PROG_NAME,
	/* max_widgets */	128,
	/* max_columns */	32,
	/* max_modules */	32,
	/* max_variables */	10000,
	/* win_width */		400,
	/* win_height */	100,
	/* layout_file */	"layout.conf",
	/* module_file */	"modules.conf",
	/* module_path */	"./modules",
	/* start_visible */	1,
	/* autohide_on_start */	2,
	/* font */		{
	/*   FONT_NORMAL */	   "-adobe-helvetica-medium-r-*-*-12-*-*-*-*-*-iso8859-2",
	/*   FONT_BOLD */	   "-adobe-helvetica-bold-r-*-*-12-*-*-*-*-*-iso8859-2",
	/*   FONT_BIG */	   "-adobe-helvetica-bold-r-*-*-24-*-*-*-*-*-iso8859-2"
				}
};

/* this MUST be sorted by constants' name */
#define CONFIG_TR_LEN	11
static P_TRANS config_tr[] = {
	/* const. name,		id, var,			type */
	{ "autohide-on-start",	11, &(cfg.autohide_on_start),	P_INTEGER },
	{ "font-big",		10, &(cfg.font[FONT_BIG]),	P_STRING },
	{ "font-bold",		 9, &(cfg.font[FONT_BOLD]),	P_STRING },
	{ "font-normal",	 8, &(cfg.font[FONT_NORMAL]),	P_STRING },
	{ "layout-file",	 5, &(cfg.layout_file),		P_STRING },
	{ "max-columns",	 2, &(cfg.max_columns),		P_INTEGER },
	{ "max-modules",	 3, &(cfg.max_modules),		P_INTEGER },
	{ "max-variables",	 4, &(cfg.max_variables),	P_INTEGER },
	{ "max-widgets",	 1, &(cfg.max_widgets),		P_INTEGER },
	{ "module-file",	 6, &(cfg.module_file),		P_STRING },
	{ "module-path",	 7, &(cfg.module_path),		P_STRING },
	{ NULL,			 0, NULL,			0 }
};

static P_FORMAT format[] = {
	P_CONSTANT,
	P_EQUAL,
	P_INTEGER | P_STRING,
	0
};

int config_load(char *filename) {
	char *d, *b, *fn;
	int r = 0;

	assert(filename != NULL);

	DBG_MSG(1, "Loading main configuration from \"%s\" ...", filename);

	d = strdup(filename);
	b = strdup(filename);
	
	chdir(dirname(d));
	fn = basename(b);

	r = parse_file(fn, NULL, format, -1, 0, 2, config_tr, CONFIG_TR_LEN);

	free(d);
	free(b);
	DBG_MSG(1, "Main configuration loaded.");
	return r;
}

