/*
 * battery.c
 * 
 * Copyright (C) 2006  Josef Kufner <jk@myserver.cz>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>

#include "battery.h"
#include "loadfile.h"

#define DEBUG 0
#include "debug.h"

#define mod_name	"battery.c"
#define MAX_LINE_LEN	80

#define FILENAME_STATUS		"/sys/class/power_supply/%s/status"
#define FILENAME_MAX_CAPACITY	"/sys/class/power_supply/%s/energy_full"
#define FILENAME_CUR_CAPACITY	"/sys/class/power_supply/%s/energy_now"
#define FILENAME_CUR_RATE	"/sys/class/power_supply/%s/current_now"
#define FILENAME_CUR_VOLTAGE	"/sys/class/power_supply/%s/voltage_now"
#define FILENAME_MIN_VOLTAGE	"/sys/class/power_supply/%s/voltage_min_design"


int read_battery(const char *battery, int *r_seconds, float *r_part, char **r_state)
{
	int capacity, rem_capacity, rate;
	char *p;

	assert(battery != NULL);
	assert(r_seconds != NULL);
	assert(r_part != NULL);
	assert(r_state != NULL);
	
	DBG_MSG(1, "Reading battery state ...");

	if (*r_state) {
		free(*r_state);
		*r_state = NULL;
	}

	/* load status */
	*r_state = loadfile(FILENAME_STATUS, battery);
	if ((p = strchr(*r_state, '\n')))
		*p = 0;

	/* load capacity & rate */
	capacity     = atoi(loadfile(FILENAME_MAX_CAPACITY, battery));
	rem_capacity = atoi(loadfile(FILENAME_CUR_CAPACITY, battery));
	rate         = atoi(loadfile(FILENAME_CUR_RATE,     battery));

	/* calculate */
	if (rem_capacity == 0) {
		*r_seconds = -1;
		*r_part = 0.0;
	} else {
		if (strcmp(*r_state, "Full") == 0) {
			*r_part = 1.0;
			*r_seconds = -1;
		} else {
			if (strcmp(*r_state, "Discharging") == 0) {
				*r_seconds = (int)((float)rem_capacity / (float)rate * 3600);
			} else {
				*r_seconds = (int)((float)(capacity - rem_capacity) / (float)rate * 3600);
			}
			*r_part = (float) rem_capacity / (float)capacity;
		}
	}

	DBG_MSG(1, "Reading battery state ... Done: seconds = %d, part = %0.1f, state = %d",
			*r_seconds, *r_part, *r_state);
	return 1;
}

