/*
 * loadfile.c
 * 
 * Copyright (C) 2007  Josef Kufner <jk@myserver.cz>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>

#include "loadfile.h"

#define DEBUG 2
#define mod_name	"battery.c"
#include "debug.h"

#define MAX_FILENAME_LEN	255
#define MAX_BUFFER_LEN		255

char *loadfile(const char *filename_fmt, ...)
{
	char *buf, *filename;
	FILE *f;
	int n;
	va_list ap;

	DBG_MSG(1, "loadfile...");

	/* prepare filename */
	va_start(ap, filename_fmt);
	filename = (char *) malloc(MAX_BUFFER_LEN);
	vsnprintf(filename, MAX_FILENAME_LEN - 1, filename_fmt, ap);
	filename[MAX_FILENAME_LEN - 1] = 0;
	va_end(ap);

	DBG_MSG(1, "Loading file \"%s\"", filename);

	/* prepare buffer */
	buf = (char *) malloc(MAX_BUFFER_LEN);
	if (!buf)
		abort();

	/* read file to buf */
	f = fopen(filename, "r");
	if (f) {
		if ((n = fread(buf, 1, MAX_BUFFER_LEN - 1, f)) <= 0) {
			perror(filename);
			buf[0] = 0;
		} else {
			buf[n] = 0;
		}
		DBG_MSG(2, "buf = \"%s\"", buf);
		if (fclose(f) == EOF)
			perror(filename);
	} else {
		perror(filename);
		buf[0] = 0;
	}

	free(filename);
	DBG_MSG(1, "Loaded data: \"%s\"", buf);
	return buf;
}

